package ActiveState::SpamEngine::IsIPV4;

use base 'Exporter';

our @EXPORT_OK = qw(
    &is_ipv4
);

sub is_ipv4 {
    my $ip = shift;
    return undef unless defined $ip;
    my @ip = split(/\./, $ip);
    if (int(@ip)==1) {
        if ($ip =~ /^0*(\d+)$/) {
            # $ip is all digits, dec/oct format
            # eg, http://2130706433/
            #     http://017700000001/
            if (length $1 < 12) {
                my $temp = $1;
                if ($temp =~ /^[1-9]/ || $temp =~ /[89]/) {
                    # dec format
                    return join('.', map { ord($_) } split(//,pack("N", $temp)));
                } else {
                    # oct format (is this even valid??)
                    return join('.', map { ord($_) } split(//,pack("N", oct($temp))));
                }
            }
        }
        elsif ($ip =~ /^0?x0*([0-9a-f]+)$/) {
            # $ip is hex format
            # eg, http://0x7f000001/
            return join('.', map { ord($_) } split(//,pack("N", hex($1)))) if length $1 < 9;
        }
        return undef;
    }
    elsif (int(@ip)==4) {
        my @bips;
        for my $byte (@ip) {
            if ($byte =~ /^([1-9]\d*)$/) {
                # dec
                # eg, 127.0.0.1
                return undef if $byte > 256;
                push @bips, $byte;
            }
            elsif ($byte =~ /^(0\d*)$/) {
                # oct, or maybe dec with leading zeros...
                if ($1 =~ /[89]/) {
                    # dec
                    # eg, 0127.0.0.1
                    return undef if $byte > 256;
                    push @bips, $byte;
                } else {
                    # might be oct
                    # eg, 0177.0.0.1
                    return undef if $byte > 377;
                    push @bips, oct($byte); # assume oct
                }
            }
            elsif ($byte =~ /^0?x0*([0-9a-f]+)$/) {
                # hex
                # eg, 0x7f.0.0.1
                return undef if length $1 > 2;
                push @bips, hex($byte);
            }
            else {
                # invalid
                return undef;
            }
        }
        # all bytes where valid
        return join('.', @bips);
    }
    return undef;
}

1;
