package ActiveState::SpamEngine::Plugin::freeweb;

use strict;

use ActiveState::SpamEngine qw(PMX_TYPE_CALLBACK PMX_TYPE_STRING
        PMX_TYPE_INTEGER DEBUG
        _get1 _getall);

use base qw(ActiveState::SpamEngine::Plugin);

our @SP_REQUIRE = qw(net part sxl);
our @SP_ATTRIB  = (
    "log-queries", PMX_TYPE_INTEGER, 0,
);

sub plugin {
    my ($eng, $name) = @_;
    for my $dep (@SP_REQUIRE) { $eng->load_plugin($dep); }
    $eng->declare_attributes(\@SP_ATTRIB, $name);
    $eng->subscribe($name, "core.compile", \&compile);
}

my ($freeweb_dbs, $match_db, $sxl, $log_queries);
sub compile {
    my $eng = shift;

    $eng->enumerate_attributes("plugin.freeweb.log-queries", \&_get1, \$log_queries);

    eval {
        $freeweb_dbs = $eng->nested_manager('freeweb.dbs');
        $match_db = $eng->nested_db('freeweb.match');
    };
    return if $@;

    $sxl = $eng->sxl('freeweb');
    $eng->subscribe('freeweb', 'part.uri', \&uri);

    $eng->initialised('freeweb');
}

sub uri {
    my $eng = shift;
    my $xtra = shift;

    unless (defined $freeweb_dbs) {
        $eng->log('WARNING', 'freeweb: no database, returning');
        return;
    }

    unless (defined $match_db) {
        $eng->log('WARNING', 'freeweb: no match database, returning');
        return;
    }

    if (DEBUG) {
        $eng->profile(3, "freeweb: known freeweb check starting");
        eval {
            require Data::Dump;
            $eng->log("DEBUG", "freeweb: freewebs to search for are:\n".Data::Dump::dump(\@_)."\n");
        };
    }

# Now do the actual searching in the DB
URI:
    for my $uri_ref (@_) {
# IMPORTANT! We must make a copy here, or any modifications we make to
# uri will modify the part list itself
        my $uri = lc $uri_ref;
        if ($uri =~ /^\w+:\/\/(.*)$/) {
            $uri = $1;
        }
       
        my ($username, $passwd, $host, $port, $path);
        my $matched;
        if ($uri =~ /^([^\\\?\@\:]+):([^\@]+)\@([\w\.-]+)((?::\d+)?)([\?\/].*)?$/) {
            ($username, $passwd, $host, $port, $path) = ($1, $2, $3, $4, $5);
        }
        elsif ($uri =~ /^([^\\\?\@]+)\@([\w\.-]+)((?::\d+)?)([\?\/].*)?$/) {
            ($username, $host, $port, $path) = ($1, $2, $3, $4);
        }
        elsif ($uri =~ /^([\w\.-]+)((?::\d+)?)([\?\/].*)?$/) {
            ($host, $port, $path) = ($1, $2, $3);
        }
        else {
            $host = $uri;
        }

        $port =~ s/^:// if $port;

        $path =~ s/\/$// if $path;

# Try uris with successively fewer leading hostnames...
        my @host_list = split /\./, $host;
        while (@host_list > 0) {
            my $domain = join('.', @host_list);
            if ($matched = $match_db->get($domain)) {
                $eng->log('DEBUG', "freeweb: found $matched domain $domain") if DEBUG;
                my ($rule, $regex) = split /\x00/, $matched;
                if ($uri =~ qr($regex)) {
                    my $capture = $1;
                    $eng->log('DEBUG', "FREEWEB QUERY LOG: [$domain] $capture ($uri)") if $log_queries;
                    my $db = $freeweb_dbs->nest($domain);
                    if($db && $db->get($capture)) {
                        $eng->addfeature($xtra, $rule, "$domain: $capture");
                        last URI;
                    }
                    elsif (!$port || $port eq '80') {
                        $capture =~ s#/#\.#g;
                        $sxl->get_uri("$capture.$domain", \&sxl_hit, [$eng, $xtra]);
                    }
                }
            }

            if ($port) {
                if ($matched = $match_db->get("$domain.$port")) {
                    $eng->log("DEBUG", "freeweb: found $matched domain $domain$port") if DEBUG;
                    
                    my ($rule, $regex) = split /\x00/, $matched;
                    if ($uri =~ qr/$regex/) {
                        my $capture = $1;
                        $eng->log('DEBUG', "FREEWEB QUERY LOG: [$domain:$port] $capture ($uri)") if $log_queries;
                        my $db = $freeweb_dbs->nest("$domain.$port");
                        if ($db && $db->get($capture)) {
                            $eng->addfeature($xtra, $rule, "$domain:$port - $capture");
                            last URI;
                        }
                        elsif ($port ne '80') {
                            $capture =~ s#/#\.#g;
                            $sxl->get_uri("$capture.$domain.$port", \&sxl_hit, [$eng, $xtra]);
                        }
                    }
                }
            }
        
            shift @host_list;
        }

    }
    $eng->profile(3, "freeweb: known freeweb check finished") if DEBUG;

}

sub sxl_hit {
    my ($x, $query, $result) = @_;
    my ($engine, $xtra) = @$x;
    $engine->addfeature($xtra, $result, $query);
}

1;
