package ActiveState::SpamEngine::Plugin::genotype;

use strict;
use ActiveState::SpamEngine qw(PMX_TYPE_INTEGER PMX_TYPE_CALLBACK
                               _get1 _getall);
use ActiveState::DB;
use ActiveState::Genotype;
use base 'ActiveState::SpamEngine::Plugin';

our @SP_REQUIRE = qw(part);
our @SP_ATTRIB  = (
    "cutoff",               PMX_TYPE_INTEGER,   0,

    # Set the PCRE recursion limit very low. Genotype is supposed to be the
    # fastest piece we have, so let's cheat as much as possible.
    #
    # Note that exact string matches use a depth of 1, so 10,000 is actually
    # quite high. The compiled-in default is 10,000,000, which is roughly 2
    # seconds of CPU time on a 2GHz P5. */
    "pcre-limit",           PMX_TYPE_INTEGER,   10000,
);

sub plugin {
    my ($eng, $name) = @_;
    for my $dep (@SP_REQUIRE) { $eng->load_plugin($dep); }
    $eng->declare_attributes(\@SP_ATTRIB, $name);
    $eng->subscribe($name, "core.compile", \&compile);
    1;
}

my $geno;
my $cutoff;
sub compile {
    my $eng = shift;
    return unless $eng->db->exists("genotype");

    # Load the compiled genotype stuff
    $eng->enumerate_attributes("plugin.genotype.cutoff", \&_get1, \$cutoff);

    $geno = eval {
        my $db = $eng->nested_db("genotype");
        ActiveState::Genotype->new($db);
    };
    if ($@) {
        $eng->log("ERROR", "genotype: can't initialise: $@");
        return;
    }

    # Log stuff to $eng->log("LEVEL", "MESSAGE");
    $geno->logger($eng);

    # Set the PCRE limit
    {
        my $l;
        $eng->enumerate_attributes("plugin.genotype.pcre-limit", \&_get1, \$l);
        $geno->pcre_limit($l) if defined $l;
    }

    $eng->subscribe("genotype", "core.meta_scan", \&geno);
    sub_fur($eng);
    $eng->initialised("genotype");
}

sub geno {
    my ($eng, $xtra, $msg) = @_;
    return if $xtra->{sum} >= $cutoff;
    my $info =$xtra->{P}{part};
    get_fur($info);
    $geno->match($eng, $xtra, $msg);
}

my $hack_fur_info = {};

sub sub_fur {

    my $engine = shift;

    $engine->subscribe('genotype','part.fur_ip',     sub { $hack_fur_info->{ip}     = $_[2] } );
    $engine->subscribe('genotype','part.fur_helo',   sub { $hack_fur_info->{helo}   = $_[2] } );
    $engine->subscribe('genotype','part.fur_header', sub { $hack_fur_info->{header} = $_[2] } );
    $engine->subscribe('genotype','part.fur_rdns',   sub { $hack_fur_info->{rdns}   = $_[2] } );
}
sub get_fur {

    my $info = $_[0];

    $info->{fur_ip} = $hack_fur_info->{ip};
    $info->{fur_helo} = $hack_fur_info->{helo};
    $info->{fur_header} = $hack_fur_info->{header};
    $info->{fur_rdns} = $hack_fur_info->{rdns};

}

1;
