package ActiveState::SpamEngine::Plugin::re;

use strict;

use ActiveState::SpamEngine qw(_get1 PMX_TYPE_INTEGER);

use base 'ActiveState::SpamEngine::Plugin';

our @SP_REQUIRE = qw(part);
our @SP_ATTRIB  = (
    "save-match",           PMX_TYPE_INTEGER,   undef,
    "pcre-limit",           PMX_TYPE_INTEGER,   undef,
);

sub plugin {
    my ($engine, $name) = @_;
    for my $dep (@SP_REQUIRE) { $engine->load_plugin($dep); }
    $engine->declare_attributes(\@SP_ATTRIB, $name);
    $engine->subscribe($name, "core.compile", \&compile);
    1;
}

my ($save, $limit);
my ($db, $db_neg);
sub compile {
    my $engine = shift;
    $engine->enumerate_attributes("plugin.re.save-match", \&_get1, \$save);
    $engine->enumerate_attributes("plugin.re.pcre-limit", \&_get1, \$limit);

    $db = eval { $engine->nested_manager("re.events.1") };
    if ($db) {
        $db->master->enumerate(sub {
            my ($engine, $event) = @_;
            $engine->subscribe('re', $event, \&handler_pos);
            1;
        }, $engine);
    }

    $db_neg = eval { $engine->nested_manager('re.events.0') };
    if ($db_neg) {
        $db_neg->master->enumerate(sub {
            my ($engine, $event) = @_;
            $engine->subscribe('re', $event, \&handler_neg);
            1;
        }, $engine);
    }

    $engine->initialised('re');
    1;
}

sub handler_pos {
    my ($engine, $extra) = splice(@_, 0, 2);
    my $event = $extra->{EVENT};
    my $def = $db->nest($event);
    $engine->pcre->match_asdb_v2(
        {
            code => \&fire,
            data => [$engine, $extra],
            logger => $engine,
            asdb => $def,
            when => 1,
            save => $save,
            limit => $limit,
        },
        @_,
    );
}

sub handler_neg {
    my ($engine, $extra) = splice(@_, 0, 2);
    my $event = $extra->{EVENT};
    my $def = $db_neg->nest($event);
    $engine->pcre->match_asdb_v2(
        {
            code => \&fire,
            data => [$engine, $extra],
            logger => $engine,
            asdb => $def,
            when => 0,
            save => 0,
            limit => $limit,
        },
        @_,
    );
}

sub fire {
    my ($data, $rule, $matched) = @_;
    my ($engine, $extra) = @$data;
    $engine->addfeature($extra, $rule, $matched);
    return 0;
}

1;
