package ActiveState::SpamEngine::Plugin::sigv2;
use strict;
use base 'ActiveState::SpamEngine::Plugin';

use ActiveState::SpamEngine qw(_get1 PMX_TYPE_INTEGER);
use Digest::MD5 2.27;

our @SP_REQUIRE = qw(part sxl);
our @SP_ATTRIB  = (
    # name,                 type,                   default,
    'log-queries',          PMX_TYPE_INTEGER,       0,
);

sub plugin {
    my ($engine, $name) = @_;
    for my $dep (@SP_REQUIRE) { $engine->load_plugin($dep); }
    $engine->declare_attributes(\@SP_ATTRIB, $name);
    $engine->subscribe($name, 'core.compile', \&compile);
    1;
}

our (
    $log_queries,
    $db_events,
    $db_dbs,
    $sxl,
);


sub compile {
    my $engine = shift;

    $engine->enumerate_attributes('plugin.sigv2.log-queries', \&_get1, \$log_queries);

    $db_events = eval { $engine->nested_manager('sigv2.events') };
    $db_dbs = eval { $engine->nested_manager('sigv2.dbs') };
    $sxl = $engine->sxl('sigv2');

    if ($db_events and $db_dbs) {
        $db_events->master->enumerate(sub {
            my ($engine, $event) = @_;
            $engine->subscribe('sigv2', $event, \&handler);
            1;
        }, $engine);
    }

    $engine->initialised('sigv2');
    1;
}

sub sxl_hit {
    my ($x, $query, $result) = @_;
    my ($engine, $xtra) = @$x;
    $engine->addfeature($xtra, $result, $query);
}

# XXX in the future, we could just use AS-Transformer to do the siggy
# stuff, essentially letting the algorithm be arbitrary

sub handler {
    my $engine = shift;
    my $extra = shift;

    my $event = $extra->{EVENT};
    my @parts = @_;

    my $db_event = eval { $db_events->nest($event) };
    if ($@) {
        $engine->log('WARNING', "sigv2: failed to get sigv2.events db for event $event.");
        return 1;
    }

    (my $part_name = $event) =~ s/^part\.//;

    my %cached_dbs = ();
    for (my $index = 0; $index < int(@parts); $index += 1) {
        my $part = $parts[$index];
        my $have_local_result = 0;

        # Don't process undef or null string events
        next if not defined $part or $part eq '';

        my $sig = create_sig($part);
        $db_event->enumerate(sub {
            my ($engine, $rule, $ignore) = @_;

            my $check_db_name = lc $rule;
            if ($log_queries) {
                $engine->log('DEBUG', "SIGV2 QUERY LOG: name=$rule part=$part_name index=$index db=$check_db_name sig=$sig");
            }

            unless ($cached_dbs{$check_db_name}) {
                $cached_dbs{$check_db_name} = $db_dbs->nest($check_db_name);
            }
            my $check_db = $cached_dbs{$check_db_name};

            if (not $check_db) {
                $engine->log('WARNING', "sigv2: failed to get $check_db_name db.");
                return 1;
            }

            if ($check_db->get($sig)) {
                $have_local_result = 1;
                $engine->addfeature($extra, $rule, $sig);
            }

            1;
        }, $engine);

        if (!$have_local_result && $sxl)
        {
            # Note that any response to the SXL query will now have to wait
            # until we receive the response.  The SXL plugin will call back to
            # the plugin if it receives a response

            $sxl->get($sig, \&sxl_hit, [$engine, $extra]);
        }
    }
}

sub create_sig {
    my $thing = shift;
    my $checksum = Digest::MD5::md5_hex($thing);
    return $checksum;
}

1;
