# spamassassin Tests to detect genuine mailing lists.

package ActiveState::SpamEngine::Plugin::spamassassin::MailingList;
1;

package ActiveState::SpamEngine::Plugin::spamassassin;

sub detect_mailing_list {
    my ($eng, $xtra, $hdrs) = @_;
    return 1 if detect_ml_ezmlm($eng, $xtra, $hdrs);
    return 1 if detect_ml_mailman($eng, $xtra, $hdrs);
    return 0;
}

# EZMLM
# Mailing-List: .*run by ezmlm
# Precedence: bulk
# List-Post: <mailto:
# List-Help: <mailto:
# List-Unsubscribe: <mailto:[a-zA-Z\.-]+-unsubscribe@
# List-Subscribe: <mailto:[a-zA-Z\.-]+-subscribe@
sub detect_ml_ezmlm {
    my ($eng, $h) = @_;
    return 0 unless h_get('mailing-list', $h) =~ /ezmlm$/;
    return 0 unless h_get('precedence', $h) eq "bulk\n";
    return 0 unless h_get('list-post', $h) =~ /^<mailto:/;
    return 0 unless h_get('list-help', $h) =~ /^<mailto:/;
    return 0 unless h_get('list-unsubscribe', $h) =~ /<mailto:[a-zA-Z\.-]+-unsubscribe\@/;
    return 0 unless h_get('list-subscribe', $h) =~ /<mailto:[a-zA-Z\.-]+-subscribe\@/;
    return 1; # assume ezmlm then.
}

# MailMan (the gnu mailing list manager)
#  Precedence: bulk
#  List-Help: <mailto:
#  List-Post: <mailto:
#  List-Subscribe: .*<mailto:.*=subscribe>
#  List-Id:
#  List-Unsubscribe: .*<mailto:.*=unsubscribe>
#  List-Archive:
#  X-Mailman-Version: \d
sub detect_ml_mailman {
    my ($eng, $xtra, $h) = @_;
    return 0 unless h_get('x-mailman-version', $h) =~ /^\d/;
    return 0 unless h_get('precedence', $h) =~ /^bulk$/;
    return 0 unless h_get('list-id', $h);
    return 0 unless h_get('list-help', $h) =~ /^<mailto:/;
    return 0 unless h_get('list-post', $h) =~ /^<mailto:/;
    return 0 unless h_get('list-subscribe', $h) =~ /<mailto:.*=subscribe>/;
    return 0 unless h_get('list-unsubscribe', $h) =~ /<mailto:.*=unsubscribe>/;
    return 0 unless h_get('list-archive', $h); # maybe comment this out.
    return 1; # assume this is a valid mailman list
}

# ListBuilder
sub detect_ml_listbuilder_headers {
    my ($eng, $xtra, $h) = @_;

    my $reply = h_get('Reply-To:addr', $h);
    if ($reply !~ /\@lb.bcentral.com/) { return 0; }

    # Received: from unknown (HELO lbrout14.listbuilder.com) (204.71.191.9)
    my $rcvd = h_get('received', $h);
    return 0 unless ($rcvd =~ /\blbrout\d+\.listbuilder\.com\b/i);
    return 0 unless ($rcvd =~ /\b204\.71\.191\.\d+\b/);

    return 1;
}

# _______________________________________________________________________
# Powered by List Builder
# To unsubscribe follow the link:
# http://lb.bcentral.com/ex/sp?c=19511&s=76CA511711046877&m=14
#
# this one is never called from detect_mailing_list()
# if we decide to call it, then __KNOWN_MAILING_LIST rule should be ROWEOB based...
sub detect_ml_listbuilder_raweob {
    my $eng  = shift;
    my $xtra = shift;
    for (@_) {
        return 0 unless /__________________{40,}\s+Powered by List Builder\s/;
        return 0 unless m#\shttp://lb\.bcentral\.com#;
        return 1;
    }
    0; # not reached
}

1;
