package ActiveState::SpamEngine::Plugin::spf;

use strict;

use ActiveState::SpamEngine qw(PMX_TYPE_CALLBACK PMX_TYPE_STRING
                               PMX_TYPE_INTEGER
                               _get1 _getall);

use base qw(ActiveState::SpamEngine::Plugin);

use Mail::SPF::Query qw();

#=============================================================================
# Support the SpamEngine plugin API
#=============================================================================
our @SP_REQUIRE = qw(net part);
our @SP_ATTRIB = (
    # KEY                           TYPE                DEFAULT
    # " -- also query header from as an attr ? -- ",             PMX_TYPE_INTEGER,   undef,
    "skip-spf-checks",  PMX_TYPE_INTEGER, 1,
);

sub plugin {
    my ($engine, $name) = @_;
    for my $dep (@SP_REQUIRE) { $engine->load_plugin($dep); }
    $engine->declare_attributes(\@SP_ATTRIB, $name);
    $engine->subscribe($name, 'core.compile', \&compile);
    return 1;
}

sub compile {
    my $engine = shift;

    my $skip;
    $engine->enumerate_attributes(
	'plugin.spf.skip-spf-checks',
	\&_get1,
	\$skip
    );
    return if $skip;
    
    my $trusted_relays;
    $engine->enumerate_attributes(
	'plugin.net.trusted-relays',
	\&_get1,
	\$trusted_relays
    );

    if ($trusted_relays) {
	# we can only use spf if trusted relays are setup.
	$engine->subscribe('spf', 'part.helo', \&process_helo);
	$engine->subscribe('spf', 'part.e_from', \&process_envelope_from);
	$engine->subscribe('spf', 'net.fur', \&process_first_untrusted_relay);
	$engine->initialised('spf');
    }

    return 1;
}

sub process_envelope_from {
    my ($engine, $extra, $envelope_from) = @_;
    my $x = $extra->{P}{spf} ||= {};
    $x->{envelope_from} = $envelope_from;
    return;
}

sub process_helo {
    my ($engine, $extra, $helo) = @_;
    my $x = $extra->{P}{spf} ||= {};
    $x->{envelope_helo} = $helo;
    # this might not be the "trusted" helo
    return;
}

sub process_first_untrusted_relay {
    my ($engine, $extra, $prh, $header) = @_;
    my $x = $extra->{P}{spf} ||= {};
	
    my $helo = $prh->{helo} || $x->{envelope_helo} || 'localhost.localdomain';
    my $envelope_from = $x->{envelope_from};

    my $query = Mail::SPF::Query->new(
	ip => $prh->{ip},
	sender=> $envelope_from,
	helo=> $helo,
	trusted=>0,
	guess=>0
    );

    my $result;
    eval q{
	$result = [ $query->result() ];
    };
=cut
	result,           # pass │ fail │ softfail │ neutral │ none │ error │ unknown [mechanism]
	smtp_comment,     # "please see http://www.openspf.org/why.html?..."  when rejecting, return this string to the SMTP client
	header_comment,   # prepend_header("Received-SPF" => "$result ($header_comment)")
	spf_record,       # "v=spf1 ..." original SPF record for the domain
=cut
    if ($@) {
	$engine->addfeature($extra, "SPF_IMPL_ERROR", $@);
    }
    else {
	my $rule_name = $result->[0];
	$rule_name =~ s/^(\S*).*$/$1/;
	$rule_name = uc $rule_name;
	$rule_name = "SPF_$rule_name";

	$engine->addfeature($extra, $rule_name, join(", ", map { $_ ? $_ : () } @$result[0..3]));
    }

    return;
}

1;
