package Sophos::Debug;
use warnings;
use strict;
use Carp;

use base 'Exporter';
our @EXPORT_OK = qw(_debug _debug_with_caller DEBUG);

our $VERSION = "0.0.1";

# example:
# echo --- test 1 ; perl -Mblib -e 'use Sophos::Debug qw(DEBUG _debug);_debug qq[foo!] if DEBUG;' ; echo --- test 2 ; PMX_SDK_DEBUG_LOGFILE=/tmp/foobar.txt perl -Mblib -e 'use Sophos::Debug qw(DEBUG _debug);_debug qq[bar!] if DEBUG;' ; cat /tmp/foobar.txt

# Detect if we're in SDK debug mode. We do this at compile time so that DEBUG
# gets optimized away most of the time.
BEGIN {
    if ( $ENV{PMX_SDK_DEBUG_LOGFILE} or $ENV{PMX_SDK_DEBUG_STDOUT} or $ENV{PMX_SDK_DEBUG_STDERR} ) {
        eval "sub DEBUG() { 1 }";
        require Time::HiRes;
        $|++;
        import Time::HiRes qw(time);
    }
    else {
        eval "sub DEBUG() { 0 }";
    }
}

sub _debug {
    my $msg = shift;
    my $caller = 1;
    if ($msg =~ /^\d$/) {
        $caller = $msg;
        $msg = shift;
    }

    _debug_with_caller([caller($caller)], $msg, @_);
}

sub _debug_with_caller {
    
    my $caller = shift;
    my $msg = shift;

    croak "Failed to pass caller array as the first parameter to _debug_with_caller()" 
        unless $caller && ref($caller) eq "ARRAY";

    my ($package, $filename, $line, $subroutine, $hasargs,
                   $wantarray, $evaltext, $is_require, $hints, $bitmask) = @$caller;

    $msg =~ s~\s+$~~gs;
    $msg =~ s~([^\s])$~$1\n~s;
    $package = "-e" if ( not defined $package );
    $subroutine = "main::" if ( not defined $subroutine );
    $subroutine =~ s~^.*::~~; # turn e.g. Sophos::LocalDNSserver::request_loop into request_loop

    my $tm = time; # Time::HiRes::time :-)
    my $t = sprintf "%s %s()", $package, $subroutine;


    my $debug_line;
    if  (scalar @_) {
        my @params;
        push(@params, (defined $_) ? $_ : 'undef') foreach (@_);
        $debug_line = sprintf "%d %f %s - $msg",$$, $tm, $t, @params;
    }
    else {
        $debug_line = sprintf "%d %f %s - %s",$$, $tm, $t, $msg;
    }

    if ( $ENV{PMX_SDK_DEBUG_LOGFILE} && $ENV{PMX_SDK_DEBUG_LOGFILE} !~ m~^\s*$~ ) {
        my $debug_file = $ENV{PMX_SDK_DEBUG_LOGFILE};

        my     $d_fd;
        open   $d_fd, ">>", $debug_file or die "failed to open the debug log file '$debug_file': $?";
        print  $d_fd $debug_line;
        close  $d_fd;
    }

    if ( $ENV{PMX_SDK_DEBUG_STDOUT} && $ENV{PMX_SDK_DEBUG_STDOUT} !~ m~^\s*$~ ) {
        print STDOUT $debug_line;
    }
    if ( $ENV{PMX_SDK_DEBUG_STDERR} && $ENV{PMX_SDK_DEBUG_STDERR} !~ m~^\s*$~ ) {
        print STDERR $debug_line;
    }

}

1;
