# 
use strict;

package Sophos::Feedback::ReportManager;

use Sophos::Feedback::Report;
use LWP::UserAgent;
use Compress::Zlib qw();
use Digest::MD5 qw(md5_hex);

# public API:
sub upload_report;
sub send_old_reports;
sub store_report;

use constant ( TIMEOUT => 60 );

# Optional arguments are:
#   proxy

sub new {
    my ($this, $stats_dir, %opts) = @_;
    my $class = ref $this || $this;
    my $self = bless {
        stats_dir => $stats_dir
    }, $class;
    $self->{$_} = $opts{$_} foreach keys %opts;
    return $self;
}

# returns the filename and the contents to upload
sub feedback_contents {
    my ($self, $report) = @_;

    my $start_time_pmx_formatted = $report->header('Report-Time') || Sophos::Feedback::Report::time2pmx_time(time);
    my $content = $report->content();
    my $compressed = Compress::Zlib::memGzip($content);
    my $md5 = md5_hex($content);

    my $license_type = $report->header('License-Type') || 'no_license';
    my $system_id = $report->header('System-Id') || 'no_systemid';
    my $report_file_name = "tr.$license_type.$system_id.$start_time_pmx_formatted.$md5.gz";
    $report_file_name =~ s/:/_/g;
    return ($report_file_name, $compressed);
}

sub upload_report {
    # Set the specified CA to be the only trusted CA.
    # # Note that this will only be used if 
    # # $upload_url is an https url.
    
    my ($self, $upload_url, $report) = @_;

    $upload_url =~ s#/$##;

    my ($report_file_name, $compressed) = $self->feedback_contents($report);

    my $ua = LWP::UserAgent->new();
    $ua->env_proxy() unless $upload_url =~ m#https://#;
    my $req = HTTP::Request->new(PUT => "$upload_url/$report_file_name");

    $req->header('Content-Type', 'application/x-gzip');

    $req->header('Content-Length', length $compressed);
    $req->content($compressed);

    my $res = $ua->request($req);

    return $res->is_success;
}

sub send_old_reports {
    # FIXME: Do nothing, because we won't be saving any old reports to disk.
}

sub store_report {
    # FIXME: Do nothing, because we don't need to store reports yet.
}
1;
