#!/tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl -w

eval 'exec /tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl -w -S $0 ${1+"$@"}'
    if 0; # not running under some shell

use strict;

if ($^O eq 'MSWin32') {
    my @dlls;
    BEGIN {
        if ($^O eq 'MSWin32') {
            require Win32;
            eval {
                my $tmpdir = PerlApp::get_temp_dir(1);
                for my $lib (qw/libeay32.dll ssleay32.dll/) {
                    my $handle = Win32::LoadLibrary("$tmpdir\\$lib");
                    push @dlls, $handle if $handle;
                }
            };
        }
    }

    END {
        if ($^O eq 'MSWin32') {
            require Win32;
            while (my $lib = shift @dlls) {
                Win32::FreeLibrary($lib);
            }
        }
    }
}

use Sophos::Feedback::ReportManager;
use Sophos::Feedback::Report;
use Getopt::Long;
use File::Basename;

use constant (DEFAULT_URL => 'https://feedback.sophos.com/asengine/v3');

sub usage {
    my ($arg0) = @_;
    my $program = $arg0 ? basename($arg0) : 'feedback';

# PMX doesn't support username/password for proxy, and I can't see how to make
# LWP support it.  When we require it, and work out how to deliver this
# feature, we'll need these options added to the API:
    print <<"EOF";
Usage: $program [OPTIONS] FEEDBACK_FILE
Options:
  --statsdir   -s    location of statistics data (core.statistics-directory).
  --proxy      -p    proxy location (e.g. webproxy.sophos.com:8080).
  --username   -n    username for proxy authentication.
  --password   -a    password for proxy authentication.
  --url --cdfs -u    HTTP URL for feedback data (CDFS).
  --cert       -c    optional Certificate Authority certificate to
                     use for authentication with server.
  --verbose    -v    display uploaded report.
  --dry-run    -d    do not actually upload report.  Implies --verbose
  --help       -h    display this help and exit.
EOF
}

my ($proxy, $statsdir, $username, $password, $url, $cert, $verbose, $dryrun);

#'username|n=s' => \$username,
#'password|a=s' => \$password,
my $result = GetOptions(
        'proxy|p=s' => \$proxy,
        'username|user|n=s' => \$username,
        'password|passwd|a=s' => \$password,
        'statsdir|s=s' => \$statsdir,
        'url|cdfs|u=s' => \$url,
        'cert|c=s' => \$cert,
        'verbose|v!' => \$verbose,
        'dry-run|d!' => \$dryrun,
        'help|h' => sub { usage($0) ; exit 0;});

# Hardwired default CDFS location
$url = DEFAULT_URL unless $url;

my %opts = ();

if ($proxy) {
    $ENV{HTTPS_PROXY} = $proxy;
    $ENV{HTTPS_PROXY_USERNAME} = $username if $username;
    $ENV{HTTPS_PROXY_PASSWORD} = $password if $password;

    if ($username && $proxy !~ m/@/) {
        $password = '' unless $password;
        if ($proxy =~ m#([A-Za-z][A-Za-z0-9]*://)(.*)$#) {
            my $prefix = $1;
            my $postfix = $2;

            # standard LWP basic authentication puts the username and password
            # into the proxy URI
            $proxy = "$prefix$username:$password\@$postfix";
        } elsif ($username) {
            $proxy = "http://$username:$password\@$proxy";
        }
    }
    $ENV{HTTP_PROXY} = $proxy;
}

$ENV{'HTTPS_CA_FILE'} = $cert if $cert;

$opts{proxy} = $proxy if $proxy;

if (!$statsdir) {
    print STDERR "Error: Location of statistics directory is required\n";
    usage($0);
    exit(1);
}

my $report_manager = Sophos::Feedback::ReportManager->new($statsdir, %opts);
my $report = Sophos::Feedback::Report->new();

$report->parse_report([<>]);

my $oldest = $report->merge_files_in_dir($statsdir, $dryrun);

if (open my $fpin, "<$statsdir/.stamp") {
    $oldest = <$fpin>;
    chomp $oldest;
    close $fpin;
}

my $now = time;
$oldest = $now - 300 unless $oldest; # default duration of 5 minutes if no timestamps were seen.
my $duration = $now - $oldest; # One day, consider rounding to the nearest 5 or 10 seconds...
$report->{headers}{'Report-Duration'} = $duration;

if (open my $fpout, ">$statsdir/.stamp") {
    print $fpout "$now\n";
    close $fpout; # if this fails, well, just ignore it because this process will be ending any moment...
}

if ($dryrun or $report_manager->upload_report($url, $report)) {
    if ($verbose or $dryrun) {
        print $report->content() . "\n" ;
    } else {
        print "success\n";
    }
    exit 0;
} else {
    print "Failed to upload report to Sophos...\n";
    exit 1;
}
